#! /usr/bin/env python
"""Compression/decompression utility using the Brotli algorithm."""

# Note: Python2 has been deprecated long ago, but some projects out in
# the wide world may still use it nevertheless. This should not
# deprive them from being able to run Brotli.
from __future__ import print_function

import argparse
import os
import platform
import sys

import brotli


# default values of encoder parameters
_DEFAULT_PARAMS = {
    'mode': brotli.MODE_GENERIC,
    'quality': 11,
    'lgwin': 22,
    'lgblock': 0,
}


def get_binary_stdio(stream):
    """Return the specified stdin/stdout/stderr stream.

    If the stdio stream requested (i.e. sys.(stdin|stdout|stderr))
    has been replaced with a stream object that does not have a `.buffer`
    attribute, this will return the original stdio stream's buffer, i.e.
    `sys.__(stdin|stdout|stderr)__.buffer`.

    Args:
      stream: One of 'stdin', 'stdout', 'stderr'.

    Returns:
      The stream, as a 'raw' buffer object (i.e. io.BufferedIOBase subclass
      instance such as io.Bufferedreader/io.BufferedWriter), suitable for
      reading/writing binary data from/to it.
    """
    if stream == 'stdin': stdio = sys.stdin
    elif stream == 'stdout': stdio = sys.stdout
    elif stream == 'stderr': stdio = sys.stderr
    else:
        raise ValueError('invalid stream name: %s' % (stream,))
    if sys.version_info[0] < 3:
        if sys.platform == 'win32':
            # set I/O stream binary flag on python2.x (Windows)
            runtime = platform.python_implementation()
            if runtime == 'PyPy':
                # the msvcrt trick doesn't work in pypy, so use fdopen().
                mode = 'rb' if stream == 'stdin' else 'wb'
                stdio = os.fdopen(stdio.fileno(), mode, 0)
            else:
                # this works with CPython -- untested on other implementations
                import msvcrt
                msvcrt.setmode(stdio.fileno(), os.O_BINARY)
        return stdio
    else:
        try:
            return stdio.buffer
        except AttributeError:
            # The Python reference explains
            # (-> https://docs.python.org/3/library/sys.html#sys.stdin)
            # that the `.buffer` attribute might not exist, since
            # the standard streams might have been replaced by something else
            # (such as an `io.StringIO()` - perhaps via
            # `contextlib.redirect_stdout()`).
            # We fall back to the original stdio in these cases.
            if stream == 'stdin': return sys.__stdin__.buffer
            if stream == 'stdout': return sys.__stdout__.buffer
            if stream == 'stderr': return sys.__stderr__.buffer
            assert False, 'Impossible Situation.'


def main(args=None):

    parser = argparse.ArgumentParser(
        prog=os.path.basename(__file__), description=__doc__)
    parser.add_argument(
        '--version', action='version', version=brotli.version)
    parser.add_argument(
        '-i',
        '--input',
        metavar='FILE',
        type=str,
        dest='infile',
        help='Input file',
        default=None)
    parser.add_argument(
        '-o',
        '--output',
        metavar='FILE',
        type=str,
        dest='outfile',
        help='Output file',
        default=None)
    parser.add_argument(
        '-f',
        '--force',
        action='store_true',
        help='Overwrite existing output file',
        default=False)
    parser.add_argument(
        '-d',
        '--decompress',
        action='store_true',
        help='Decompress input file',
        default=False)
    params = parser.add_argument_group('optional encoder parameters')
    params.add_argument(
        '-m',
        '--mode',
        metavar='MODE',
        type=int,
        choices=[0, 1, 2],
        help='The compression mode can be 0 for generic input, '
        '1 for UTF-8 encoded text, or 2 for WOFF 2.0 font data. '
        'Defaults to 0.')
    params.add_argument(
        '-q',
        '--quality',
        metavar='QUALITY',
        type=int,
        choices=list(range(0, 12)),
        help='Controls the compression-speed vs compression-density '
        'tradeoff. The higher the quality, the slower the '
        'compression. Range is 0 to 11. Defaults to 11.')
    params.add_argument(
        '--lgwin',
        metavar='LGWIN',
        type=int,
        choices=list(range(10, 25)),
        help='Base 2 logarithm of the sliding window size. Range is '
        '10 to 24. Defaults to 22.')
    params.add_argument(
        '--lgblock',
        metavar='LGBLOCK',
        type=int,
        choices=[0] + list(range(16, 25)),
        help='Base 2 logarithm of the maximum input block size. '
        'Range is 16 to 24. If set to 0, the value will be set based '
        'on the quality. Defaults to 0.')
    # set default values using global _DEFAULT_PARAMS dictionary
    parser.set_defaults(**_DEFAULT_PARAMS)

    options = parser.parse_args(args=args)

    if options.infile:
        try:
            with open(options.infile, 'rb') as infile:
                data = infile.read()
        except OSError:
            parser.error('Could not read --infile: %s' % (infile,))
    else:
        if sys.stdin.isatty():
            # interactive console, just quit
            parser.error('No input (called from interactive terminal).')
        infile = get_binary_stdio('stdin')
        data = infile.read()

    if options.outfile:
        # Caution! If `options.outfile` is a broken symlink, will try to
        # redirect the write according to symlink.
        if os.path.exists(options.outfile) and not options.force:
            parser.error(('Target --outfile=%s already exists, '
                          'but --force was not requested.') % (options.outfile,))
        outfile = open(options.outfile, 'wb')
        did_open_outfile = True
    else:
        outfile = get_binary_stdio('stdout')
        did_open_outfile = False
    try:
        try:
            if options.decompress:
                data = brotli.decompress(data)
            else:
                data = brotli.compress(
                    data,
                    mode=options.mode,
                    quality=options.quality,
                    lgwin=options.lgwin,
                    lgblock=options.lgblock)
            outfile.write(data)
        finally:
            if did_open_outfile: outfile.close()
    except brotli.error as e:
        parser.exit(1,
                    'bro: error: %s: %s' % (e, options.infile or '{stdin}'))


if __name__ == '__main__':
    main()



# Copyright 2016 The Brotli Authors. All rights reserved.
#
# Distributed under MIT license.
# See file LICENSE for detail or copy at https://opensource.org/licenses/MIT

import unittest

import brotli

from . import _test_utils


def _get_original_name(test_data):
  return test_data.split('.compressed')[0]


class TestDecompress(_test_utils.TestCase):

  def _check_decompression(self, test_data):
    # Verify decompression matches the original.
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    original = _get_original_name(test_data)
    self.assert_files_match(temp_uncompressed, original)

  def _decompress(self, test_data):
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    with open(temp_uncompressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        out_file.write(brotli.decompress(in_file.read()))

  def _test_decompress(self, test_data):
    self._decompress(test_data)
    self._check_decompression(test_data)

  def test_garbage_appended(self):
    with self.assertRaises(brotli.error):
      brotli.decompress(brotli.compress(b'a') + b'a')


_test_utils.generate_test_methods(TestDecompress, for_decompression=True)

if __name__ == '__main__':
  unittest.main()



# Copyright 2016 The Brotli Authors. All rights reserved.
#
# Distributed under MIT license.
# See file LICENSE for detail or copy at https://opensource.org/licenses/MIT

import subprocess
import unittest

from . import _test_utils

BRO_ARGS = _test_utils.BRO_ARGS
TEST_ENV = _test_utils.TEST_ENV


def _get_original_name(test_data):
  return test_data.split('.compressed')[0]


class TestBroDecompress(_test_utils.TestCase):

  def _check_decompression(self, test_data):
    # Verify decompression matches the original.
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    original = _get_original_name(test_data)
    self.assert_files_match(temp_uncompressed, original)

  def _decompress_file(self, test_data):
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    args = BRO_ARGS + ['-f', '-d', '-i', test_data, '-o', temp_uncompressed]
    subprocess.check_call(args, env=TEST_ENV)

  def _decompress_pipe(self, test_data):
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    args = BRO_ARGS + ['-d']
    with open(temp_uncompressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        subprocess.check_call(
            args, stdin=in_file, stdout=out_file, env=TEST_ENV
        )

  def _test_decompress_file(self, test_data):
    self._decompress_file(test_data)
    self._check_decompression(test_data)

  def _test_decompress_pipe(self, test_data):
    self._decompress_pipe(test_data)
    self._check_decompression(test_data)


_test_utils.generate_test_methods(TestBroDecompress, for_decompression=True)


class TestBroCompress(_test_utils.TestCase):

  VARIANTS = {'quality': (1, 6, 9, 11), 'lgwin': (10, 15, 20, 24)}

  def _check_decompression(self, test_data):
    # Write decompression to temp file and verify it matches the original.
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    original = test_data
    args = BRO_ARGS + ['-f', '-d']
    args.extend(['-i', temp_compressed, '-o', temp_uncompressed])
    subprocess.check_call(args, env=TEST_ENV)
    self.assert_files_match(temp_uncompressed, original)

  def _compress_file(self, test_data, **kwargs):
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    args = BRO_ARGS + ['-f']
    if 'quality' in kwargs:
      args.extend(['-q', str(kwargs['quality'])])
    if 'lgwin' in kwargs:
      args.extend(['--lgwin', str(kwargs['lgwin'])])
    args.extend(['-i', test_data, '-o', temp_compressed])
    subprocess.check_call(args, env=TEST_ENV)

  def _compress_pipe(self, test_data, **kwargs):
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    args = BRO_ARGS
    if 'quality' in kwargs:
      args.extend(['-q', str(kwargs['quality'])])
    if 'lgwin' in kwargs:
      args.extend(['--lgwin', str(kwargs['lgwin'])])
    with open(temp_compressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        subprocess.check_call(
            args, stdin=in_file, stdout=out_file, env=TEST_ENV
        )

  def _test_compress_file(self, test_data, **kwargs):
    self._compress_file(test_data, **kwargs)
    self._check_decompression(test_data)

  def _test_compress_pipe(self, test_data, **kwargs):
    self._compress_pipe(test_data, **kwargs)
    self._check_decompression(test_data)


_test_utils.generate_test_methods(
    TestBroCompress, variants=TestBroCompress.VARIANTS
)

if __name__ == '__main__':
  unittest.main()



"""Common utilities for Brotli tests."""

from __future__ import print_function
import filecmp
import glob
import itertools
import os
import sys
import sysconfig
import tempfile
import unittest

# TODO(eustas): use str(pathlib.PurePath(file).parent.parent) for Python 3.4+
project_dir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
test_dir = os.getenv('BROTLI_TESTS_PATH')
BRO_ARGS = [os.getenv('BROTLI_WRAPPER')]

# Fallbacks
if test_dir is None:
  test_dir = os.path.join(project_dir, 'tests')
if BRO_ARGS[0] is None:
  python_exe = sys.executable or 'python'
  bro_path = os.path.join(project_dir, 'python', 'bro.py')
  BRO_ARGS = [python_exe, bro_path]

# Get the platform/version-specific build folder.
# By default, the distutils build base is in the same location as setup.py.
platform_lib_name = 'lib.{platform}-{version[0]}.{version[1]}'.format(
    platform=sysconfig.get_platform(), version=sys.version_info
)
build_dir = os.path.join(project_dir, 'bin', platform_lib_name)

# Prepend the build folder to sys.path and the PYTHONPATH environment variable.
if build_dir not in sys.path:
  sys.path.insert(0, build_dir)
TEST_ENV = dict(os.environ)
if 'PYTHONPATH' not in TEST_ENV:
  TEST_ENV['PYTHONPATH'] = build_dir
else:
  TEST_ENV['PYTHONPATH'] = build_dir + os.pathsep + TEST_ENV['PYTHONPATH']

TESTDATA_DIR = os.path.join(test_dir, 'testdata')

TESTDATA_FILES = [
    'empty',  # Empty file
    '10x10y',  # Small text
    'alice29.txt',  # Large text
    'random_org_10k.bin',  # Small data
    'mapsdatazrh',  # Large data
    'ukkonooa',  # Poem
    'cp1251-utf16le',  # Codepage 1251 table saved in UTF16-LE encoding
    'cp852-utf8',  # Codepage 852 table saved in UTF8 encoding
    # TODO(eustas): add test on already compressed content
]

# Some files might be missing in a lightweight sources pack.
TESTDATA_PATH_CANDIDATES = [
    os.path.join(TESTDATA_DIR, f) for f in TESTDATA_FILES
]

TESTDATA_PATHS = [
    path for path in TESTDATA_PATH_CANDIDATES if os.path.isfile(path)
]

TESTDATA_PATHS_FOR_DECOMPRESSION = glob.glob(
    os.path.join(TESTDATA_DIR, '*.compressed')
)

TEMP_DIR = tempfile.mkdtemp()


def get_temp_compressed_name(filename):
  return os.path.join(TEMP_DIR, os.path.basename(filename + '.bro'))


def get_temp_uncompressed_name(filename):
  return os.path.join(TEMP_DIR, os.path.basename(filename + '.unbro'))


def bind_method_args(method, *args, **kwargs):
  return lambda self: method(self, *args, **kwargs)


# TODO(eustas): migrate to absl.testing.parameterized.
def generate_test_methods(
    test_case_class, for_decompression=False, variants=None
):
  """Adds test methods for each test data file and each variant.

  This makes identifying problems with specific compression scenarios easier.

  Args:
    test_case_class: The test class to add methods to.
    for_decompression: If True, uses compressed test data files.
    variants: A dictionary where keys are option names and values are lists of
      possible values for that option. Each combination of variants will
      generate a separate test method.
  """
  if for_decompression:
    paths = [
        path for path in TESTDATA_PATHS_FOR_DECOMPRESSION
        if os.path.exists(path.replace('.compressed', ''))
    ]
  else:
    paths = TESTDATA_PATHS
  opts = []
  if variants:
    opts_list = []
    for k, v in variants.items():
      opts_list.append([r for r in itertools.product([k], v)])
    for o in itertools.product(*opts_list):
      opts_name = '_'.join([str(i) for i in itertools.chain(*o)])
      opts_dict = dict(o)
      opts.append([opts_name, opts_dict])
  else:
    opts.append(['', {}])
  for method in [m for m in dir(test_case_class) if m.startswith('_test')]:
    for testdata in paths:
      for opts_name, opts_dict in opts:
        f = os.path.splitext(os.path.basename(testdata))[0]
        name = 'test_{method}_{options}_{file}'.format(
            method=method, options=opts_name, file=f
        )
        func = bind_method_args(
            getattr(test_case_class, method), testdata, **opts_dict
        )
        setattr(test_case_class, name, func)


class TestCase(unittest.TestCase):
  """Base class for Brotli test cases.

  Provides common setup and teardown logic, including cleaning up temporary
  files and a utility for comparing file contents.
  """

  def tearDown(self):
    for f in TESTDATA_PATHS:
      try:
        os.unlink(get_temp_compressed_name(f))
      except OSError:
        pass
      try:
        os.unlink(get_temp_uncompressed_name(f))
      except OSError:
        pass
    # super().tearDown()  # Requires Py3+

  def assert_files_match(self, first, second):
    self.assertTrue(
        filecmp.cmp(first, second, shallow=False),
        'File {} differs from {}'.format(first, second),
    )



# Copyright 2016 The Brotli Authors. All rights reserved.
#
# Distributed under MIT license.
# See file LICENSE for detail or copy at https://opensource.org/licenses/MIT

import functools
import os
import unittest

import brotli

from . import _test_utils


def _get_original_name(test_data):
  return test_data.split('.compressed')[0]


class TestDecompressor(_test_utils.TestCase):

  CHUNK_SIZE = 1
  MIN_OUTPUT_BUFFER_SIZE = 32768  # Actually, several bytes less.

  def setUp(self):
    # super().setUp()  # Requires Py3+
    self.decompressor = brotli.Decompressor()

  def tearDown(self):
    self.decompressor = None
    # super().tearDown()  # Requires Py3+

  def _check_decompression(self, test_data):
    # Verify decompression matches the original.
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    original = _get_original_name(test_data)
    self.assert_files_match(temp_uncompressed, original)

  def _decompress(self, test_data):
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    with open(temp_uncompressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        read_chunk = functools.partial(in_file.read, self.CHUNK_SIZE)
        for data in iter(read_chunk, b''):
          out_file.write(self.decompressor.process(data))
    self.assertTrue(self.decompressor.is_finished())

  def _decompress_with_limit(self, test_data):
    output_buffer_limit = 10922
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    with open(temp_uncompressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        chunk_iter = iter(functools.partial(in_file.read, 10 * 1024), b'')
        while not self.decompressor.is_finished():
          data = b''
          if self.decompressor.can_accept_more_data():
            data = next(chunk_iter, b'')
          decompressed_data = self.decompressor.process(
              data, output_buffer_limit=output_buffer_limit
          )
          self.assertLessEqual(
              len(decompressed_data), self.MIN_OUTPUT_BUFFER_SIZE
          )
          out_file.write(decompressed_data)
        self.assertIsNone(next(chunk_iter, None))

  def _test_decompress(self, test_data):
    self._decompress(test_data)
    self._check_decompression(test_data)

  def _test_decompress_with_limit(self, test_data):
    self._decompress_with_limit(test_data)
    self._check_decompression(test_data)

  def test_too_much_input(self):
    with open(
        os.path.join(_test_utils.TESTDATA_DIR, 'zerosukkanooa.compressed'), 'rb'
    ) as in_file:
      compressed = in_file.read()
      self.decompressor.process(compressed[:-1], output_buffer_limit=10240)
      # the following assertion checks whether the test setup is correct
      self.assertFalse(self.decompressor.can_accept_more_data())
      with self.assertRaises(brotli.error):
        self.decompressor.process(compressed[-1:])

  def test_changing_limit(self):
    test_data = os.path.join(
        _test_utils.TESTDATA_DIR, 'zerosukkanooa.compressed'
    )
    check_output = os.path.exists(test_data.replace('.compressed', ''))
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    with open(temp_uncompressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        compressed = in_file.read()
        uncompressed = self.decompressor.process(
            compressed[:-1], output_buffer_limit=10240
        )
        self.assertLessEqual(len(uncompressed), self.MIN_OUTPUT_BUFFER_SIZE)
        out_file.write(uncompressed)
        while not self.decompressor.can_accept_more_data():
          out_file.write(self.decompressor.process(b''))
        out_file.write(self.decompressor.process(compressed[-1:]))
    if check_output:
      self._check_decompression(test_data)

  def test_garbage_appended(self):
    with self.assertRaises(brotli.error):
      self.decompressor.process(brotli.compress(b'a') + b'a')

  def test_already_finished(self):
    self.decompressor.process(brotli.compress(b'a'))
    with self.assertRaises(brotli.error):
      self.decompressor.process(b'a')


_test_utils.generate_test_methods(TestDecompressor, for_decompression=True)

if __name__ == '__main__':
  unittest.main()



# Copyright 2016 The Brotli Authors. All rights reserved.
#
# Distributed under MIT license.
# See file LICENSE for detail or copy at https://opensource.org/licenses/MIT

import unittest

import brotli

from . import _test_utils


class TestCompress(_test_utils.TestCase):

  VARIANTS = {'quality': (1, 6, 9, 11), 'lgwin': (10, 15, 20, 24)}

  def _check_decompression(self, test_data):
    # Write decompression to temp file and verify it matches the original.
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    original = test_data
    with open(temp_uncompressed, 'wb') as out_file:
      with open(temp_compressed, 'rb') as in_file:
        out_file.write(brotli.decompress(in_file.read()))
    self.assert_files_match(temp_uncompressed, original)

  def _compress(self, test_data, **kwargs):
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    with open(temp_compressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        out_file.write(brotli.compress(in_file.read(), **kwargs))

  def _test_compress(self, test_data, **kwargs):
    self._compress(test_data, **kwargs)
    self._check_decompression(test_data)


_test_utils.generate_test_methods(TestCompress, variants=TestCompress.VARIANTS)

if __name__ == '__main__':
  unittest.main()



# Copyright 2015 The Brotli Authors. All rights reserved.
#
# Distributed under MIT license.
# See file LICENSE for detail or copy at https://opensource.org/licenses/MIT

import os
import platform
import re
import unittest

try:
    from setuptools import Extension
    from setuptools import setup
except:
    from distutils.core import Extension
    from distutils.core import setup
from distutils.command.build_ext import build_ext
from distutils import errors
from distutils import dep_util
from distutils import log



CURR_DIR = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))


def bool_from_environ(key):
    value = os.environ.get(key)
    if not value:
        return False
    if value == "1":
       return True
    if value == "0":
       return False
    raise ValueError("Environment variable {} has invalid value {}. Please set it to 1, 0 or an empty string".format(key, value))


def read_define(path, macro):
    """Return macro value from the given file."""
    with open(path, "r") as f:
        for line in f:
            m = re.match(r"#define\s{}\s+(.+)".format(macro), line)
            if m:
                return m.group(1)
    return ""


def get_version():
    """Return library version string from 'common/version.h' file."""
    version_file_path = os.path.join(CURR_DIR, "c", "common", "version.h")
    major = read_define(version_file_path, "BROTLI_VERSION_MAJOR")
    minor = read_define(version_file_path, "BROTLI_VERSION_MINOR")
    patch = read_define(version_file_path, "BROTLI_VERSION_PATCH")
    if not major or not minor or not patch:
        return ""
    return "{}.{}.{}".format(major, minor, patch)


def get_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover("python", pattern="*_test.py")
    return test_suite


class BuildExt(build_ext):
    def get_source_files(self):
        filenames = build_ext.get_source_files(self)
        for ext in self.extensions:
            filenames.extend(ext.depends)
        return filenames

    def build_extension(self, ext):
        if ext.sources is None or not isinstance(ext.sources, (list, tuple)):
            raise errors.DistutilsSetupError(
                "in 'ext_modules' option (extension '%s'), "
                "'sources' must be present and must be "
                "a list of source filenames" % ext.name
            )

        ext_path = self.get_ext_fullpath(ext.name)
        depends = ext.sources + ext.depends
        if not (self.force or dep_util.newer_group(depends, ext_path, "newer")):
            log.debug("skipping '%s' extension (up-to-date)", ext.name)
            return
        else:
            log.info("building '%s' extension", ext.name)

        c_sources = []
        for source in ext.sources:
            if source.endswith(".c"):
                c_sources.append(source)
        extra_args = ext.extra_compile_args or []

        objects = []

        macros = ext.define_macros[:]
        if self.compiler.compiler_type == "mingw32":
            # On Windows Python 2.7, pyconfig.h defines "hypot" as "_hypot",
            # This clashes with GCC's cmath, and causes compilation errors when
            # building under MinGW: http://bugs.python.org/issue11566
            macros.append(("_hypot", "hypot"))
        for undef in ext.undef_macros:
            macros.append((undef,))

        objs = self.compiler.compile(
            c_sources,
            output_dir=self.build_temp,
            macros=macros,
            include_dirs=ext.include_dirs,
            debug=self.debug,
            extra_postargs=extra_args,
            depends=ext.depends,
        )
        objects.extend(objs)

        self._built_objects = objects[:]
        if ext.extra_objects:
            objects.extend(ext.extra_objects)
        extra_args = ext.extra_link_args or []
        # when using GCC on Windows, we statically link libgcc and libstdc++,
        # so that we don't need to package extra DLLs
        if self.compiler.compiler_type == "mingw32":
            extra_args.extend(["-static-libgcc", "-static-libstdc++"])

        ext_path = self.get_ext_fullpath(ext.name)
        # Detect target language, if not provided
        language = ext.language or self.compiler.detect_language(c_sources)

        self.compiler.link_shared_object(
            objects,
            ext_path,
            libraries=self.get_libraries(ext),
            library_dirs=ext.library_dirs,
            runtime_library_dirs=ext.runtime_library_dirs,
            extra_postargs=extra_args,
            export_symbols=self.get_export_symbols(ext),
            debug=self.debug,
            build_temp=self.build_temp,
            target_lang=language,
        )


NAME = "Brotli"

VERSION = get_version()

URL = "https://github.com/google/brotli"

DESCRIPTION = "Python bindings for the Brotli compression library"

AUTHOR = "The Brotli Authors"

LICENSE = "MIT"

PLATFORMS = ["Posix", "MacOS X", "Windows"]

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: Developers",
    # Deprecated, see https://packaging.python.org/en/latest/guides/writing-pyproject-toml/#license for details.
    # "License :: OSI Approved :: MIT License",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: POSIX :: Linux",
    "Programming Language :: C",
    "Programming Language :: C++",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Unix Shell",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: System :: Archiving",
    "Topic :: System :: Archiving :: Compression",
    "Topic :: Text Processing :: Fonts",
    "Topic :: Utilities",
]

PACKAGE_DIR = {"": "python"}

PY_MODULES = ["brotli"]

USE_SYSTEM_BROTLI = bool_from_environ("USE_SYSTEM_BROTLI")

if USE_SYSTEM_BROTLI:
    import pkgconfig
    
    REQUIRED_BROTLI_SYSTEM_LIBRARIES = ["libbrotlicommon", "libbrotlienc", "libbrotlidec"]

    define_macros = []
    include_dirs = []
    libraries = []
    library_dirs = []

    for required_system_library in REQUIRED_BROTLI_SYSTEM_LIBRARIES:
        package_configuration = pkgconfig.parse(required_system_library)

        define_macros += package_configuration["define_macros"]
        include_dirs += package_configuration["include_dirs"]
        libraries += package_configuration["libraries"]
        library_dirs += package_configuration["library_dirs"]

    brotli_extension = Extension(
        '_brotli',
        sources=[
            'python/_brotli.c'
        ],
        include_dirs=include_dirs,
        define_macros=define_macros,
        libraries=libraries,
        library_dirs=library_dirs
    )


    EXT_MODULES = [brotli_extension]
else:
    EXT_MODULES = [
        Extension(
            "_brotli",
            sources=[
                "python/_brotli.c",
                "c/common/constants.c",
                "c/common/context.c",
                "c/common/dictionary.c",
                "c/common/platform.c",
                "c/common/shared_dictionary.c",
                "c/common/transform.c",
                "c/dec/bit_reader.c",
                "c/dec/decode.c",
                "c/dec/huffman.c",
                "c/dec/prefix.c",
                "c/dec/state.c",
                "c/dec/static_init.c",
                "c/enc/backward_references.c",
                "c/enc/backward_references_hq.c",
                "c/enc/bit_cost.c",
                "c/enc/block_splitter.c",
                "c/enc/brotli_bit_stream.c",
                "c/enc/cluster.c",
                "c/enc/command.c",
                "c/enc/compound_dictionary.c",
                "c/enc/compress_fragment.c",
                "c/enc/compress_fragment_two_pass.c",
                "c/enc/dictionary_hash.c",
                "c/enc/encode.c",
                "c/enc/encoder_dict.c",
                "c/enc/entropy_encode.c",
                "c/enc/fast_log.c",
                "c/enc/histogram.c",
                "c/enc/literal_cost.c",
                "c/enc/memory.c",
                "c/enc/metablock.c",
                "c/enc/static_dict.c",
                "c/enc/static_dict_lut.c",
                "c/enc/static_init.c",
                "c/enc/utf8_util.c",
            ],
            depends=[
                "c/common/constants.h",
                "c/common/context.h",
                "c/common/dictionary.h",
                "c/common/platform.h",
                "c/common/shared_dictionary_internal.h",
                "c/common/static_init.h",
                "c/common/transform.h",
                "c/common/version.h",
                "c/dec/bit_reader.h",
                "c/dec/huffman.h",
                "c/dec/prefix.h",
                "c/dec/prefix_inc.h",
                "c/dec/state.h",
                "c/dec/static_init.h",
                "c/enc/backward_references.h",
                "c/enc/backward_references_hq.h",
                "c/enc/backward_references_inc.h",
                "c/enc/bit_cost.h",
                "c/enc/bit_cost_inc.h",
                "c/enc/block_encoder_inc.h",
                "c/enc/block_splitter.h",
                "c/enc/block_splitter_inc.h",
                "c/enc/brotli_bit_stream.h",
                "c/enc/cluster.h",
                "c/enc/cluster_inc.h",
                "c/enc/command.h",
                "c/enc/compound_dictionary.h",
                "c/enc/compress_fragment.h",
                "c/enc/compress_fragment_two_pass.h",
                "c/enc/dictionary_hash.h",
                "c/enc/dictionary_hash_inc.h",
                "c/enc/encoder_dict.h",
                "c/enc/entropy_encode.h",
                "c/enc/entropy_encode_static.h",
                "c/enc/fast_log.h",
                "c/enc/find_match_length.h",
                "c/enc/hash.h",
                "c/enc/hash_composite_inc.h",
                "c/enc/hash_forgetful_chain_inc.h",
                "c/enc/hash_longest_match64_inc.h",
                "c/enc/hash_longest_match_inc.h",
                "c/enc/hash_longest_match_quickly_inc.h",
                "c/enc/hash_rolling_inc.h",
                "c/enc/hash_to_binary_tree_inc.h",
                "c/enc/histogram.h",
                "c/enc/histogram_inc.h",
                "c/enc/literal_cost.h",
                "c/enc/memory.h",
                "c/enc/metablock.h",
                "c/enc/metablock_inc.h",
                "c/enc/params.h",
                "c/enc/prefix.h",
                "c/enc/quality.h",
                "c/enc/ringbuffer.h",
                "c/enc/static_dict.h",
                "c/enc/static_dict_lut.h",
                "c/enc/static_init.h",
                "c/enc/utf8_util.h",
                "c/enc/write_bits.h",
            ],
            include_dirs=[
                "c/include",
            ]),
    ]

TEST_SUITE = "setup.get_test_suite"

CMD_CLASS = {
    "build_ext": BuildExt,
}

with open("README.md", "r") as f:
    README = f.read()

setup(
    name=NAME,
    description=DESCRIPTION,
    long_description=README,
    long_description_content_type="text/markdown",
    version=VERSION,
    url=URL,
    author=AUTHOR,
    license=LICENSE,
    platforms=PLATFORMS,
    classifiers=CLASSIFIERS,
    package_dir=PACKAGE_DIR,
    py_modules=PY_MODULES,
    ext_modules=EXT_MODULES,
    test_suite=TEST_SUITE,
    cmdclass=CMD_CLASS,
)



# Copyright 2016 The Brotli Authors. All rights reserved.
#
# Distributed under MIT license.
# See file LICENSE for detail or copy at https://opensource.org/licenses/MIT

import functools
import unittest

import brotli

from . import _test_utils


# Do not inherit from TestCase here to ensure that test methods
# are not run automatically and instead are run as part of a specific
# configuration below.
class _TestCompressor(object):

  CHUNK_SIZE = 2048

  def tearDown(self):
    self.compressor = None
    # super().tearDown()  # Requires Py3+

  def _check_decompression(self, test_data):
    # Write decompression to temp file and verify it matches the original.
    temp_uncompressed = _test_utils.get_temp_uncompressed_name(test_data)
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    original = test_data
    with open(temp_uncompressed, 'wb') as out_file:
      with open(temp_compressed, 'rb') as in_file:
        out_file.write(brotli.decompress(in_file.read()))
    self.assert_files_match(temp_uncompressed, original)

  def _test_single_process(self, test_data):
    # Write single-shot compression to temp file.
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    with open(temp_compressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        out_file.write(self.compressor.process(in_file.read()))
      out_file.write(self.compressor.finish())
    self._check_decompression(test_data)

  def _test_multiple_process(self, test_data):
    # Write chunked compression to temp file.
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    with open(temp_compressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        read_chunk = functools.partial(in_file.read, self.CHUNK_SIZE)
        for data in iter(read_chunk, b''):
          out_file.write(self.compressor.process(data))
      out_file.write(self.compressor.finish())
    self._check_decompression(test_data)

  def _test_multiple_process_and_flush(self, test_data):
    # Write chunked and flushed compression to temp file.
    temp_compressed = _test_utils.get_temp_compressed_name(test_data)
    with open(temp_compressed, 'wb') as out_file:
      with open(test_data, 'rb') as in_file:
        read_chunk = functools.partial(in_file.read, self.CHUNK_SIZE)
        for data in iter(read_chunk, b''):
          out_file.write(self.compressor.process(data))
          out_file.write(self.compressor.flush())
      out_file.write(self.compressor.finish())
    self._check_decompression(test_data)


_test_utils.generate_test_methods(_TestCompressor)


class TestCompressorQuality1(_TestCompressor, _test_utils.TestCase):

  def setUp(self):
    # super().setUp()  # Requires Py3+
    self.compressor = brotli.Compressor(quality=1)


class TestCompressorQuality6(_TestCompressor, _test_utils.TestCase):

  def setUp(self):
    # super().setUp()  # Requires Py3+
    self.compressor = brotli.Compressor(quality=6)


class TestCompressorQuality9(_TestCompressor, _test_utils.TestCase):

  def setUp(self):
    # super().setUp()  # Requires Py3+
    self.compressor = brotli.Compressor(quality=9)


class TestCompressorQuality11(_TestCompressor, _test_utils.TestCase):

  def setUp(self):
    # super().setUp()  # Requires Py3+
    self.compressor = brotli.Compressor(quality=11)


if __name__ == '__main__':
  unittest.main()
